@@ -30,6 +30,15 @@ func Example() {
 		panic(err)
 	}
 
-	fmt.Println(doc.Paths["/foo"].Get.Responses["200"].Value.Content["application/json"].Schema.Value.Properties["foo2"].Value.Properties["foo"].Value.Properties["bar"].Value.Type)
-	// Output: string
+	fmt.Println(doc.
+		Paths.Value("/foo").
+		Get.Responses.Value("200").Value.
+		Content["application/json"].
+		Schema.Value.
+		Properties["foo2"].Value.
+		Properties["foo"].Value.
+		Properties["bar"].Value.
+		Type,
+	)
+	// Output: &[string]
 }
