@@ -22,6 +22,11 @@ func TestIssue220(t *testing.T) {
 		err = doc.Validate(loader.Context)
 		require.NoError(t, err)
 
-		require.Equal(t, "integer", doc.Paths["/foo"].Get.Responses["200"].Value.Content["application/json"].Schema.Value.Properties["bar"].Value.Type)
+		require.Equal(t, &Types{"integer"}, doc.
+			Paths.Value("/foo").
+			Get.Responses.Value("200").Value.
+			Content["application/json"].
+			Schema.Value.Properties["bar"].Value.
+			Type)
 	}
 }
