@@ -1,6 +1,7 @@
 package openapi3
 
 import (
+	"bytes"
 	"fmt"
 	"net"
 	"net/http"
@@ -60,8 +61,8 @@ paths:
 	require.NoError(t, err)
 	require.Equal(t, "An API", doc.Info.Title)
 	require.Equal(t, 2, len(doc.Components.Schemas))
-	require.Equal(t, 1, len(doc.Paths))
-	require.Equal(t, "unexpected error", *doc.Paths["/items"].Put.Responses.Default().Value.Description)
+	require.Equal(t, 1, doc.Paths.Len())
+	require.Equal(t, "unexpected error", *doc.Paths.Value("/items").Put.Responses.Default().Value.Description)
 
 	err = doc.Validate(loader.Context)
 	require.NoError(t, err)
@@ -103,11 +104,25 @@ paths:
 }
 
 func ExampleLoader() {
-	const source = `{"info":{"description":"An API"}}`
-	doc, err := NewLoader().LoadFromData([]byte(source))
+	spec := []byte(`
+openapi: 3.0.1
+paths: {}
+info:
+  version: 1.1.1
+  title: title
+  description: An API
+`[1:])
+
+	loader := NewLoader()
+	doc, err := loader.LoadFromData(spec)
 	if err != nil {
 		panic(err)
 	}
+
+	if err := doc.Validate(loader.Context); err != nil {
+		panic(err)
+	}
+
 	fmt.Print(doc.Info.Description)
 	// Output: An API
 }
@@ -154,7 +169,7 @@ paths:
 	err = doc.Validate(loader.Context)
 	require.NoError(t, err)
 
-	example := doc.Paths["/"].Get.Responses.Get(200).Value.Content.Get("application/json").Examples["test"]
+	example := doc.Paths.Value("/").Get.Responses.Status(200).Value.Content.Get("application/json").Examples["test"]
 	require.NotNil(t, example.Value)
 	require.Equal(t, example.Value.Value.(map[string]interface{})["error"].(bool), false)
 }
@@ -217,7 +232,7 @@ paths:
 	doc, err := loader.LoadFromData(spec)
 	require.NoError(t, err)
 
-	require.NotNil(t, doc.Paths["/"].Parameters[0].Value)
+	require.NotNil(t, doc.Paths.Value("/").Parameters[0].Value)
 }
 
 func TestLoadRequestExampleRef(t *testing.T) {
@@ -249,7 +264,7 @@ paths:
 	doc, err := loader.LoadFromData(spec)
 	require.NoError(t, err)
 
-	require.NotNil(t, doc.Paths["/"].Post.RequestBody.Value.Content.Get("application/json").Examples["test"])
+	require.NotNil(t, doc.Paths.Value("/").Post.RequestBody.Value.Content.Get("application/json").Examples["test"])
 }
 
 func createTestServer(t *testing.T, handler http.Handler) *httptest.Server {
@@ -275,7 +290,7 @@ func TestLoadFromRemoteURL(t *testing.T) {
 	doc, err := loader.LoadFromURI(url)
 	require.NoError(t, err)
 
-	require.Equal(t, "string", doc.Components.Schemas["TestSchema"].Value.Type)
+	require.Equal(t, &Types{"string"}, doc.Components.Schemas["TestSchema"].Value.Type)
 }
 
 func TestLoadWithReferenceInReference(t *testing.T) {
@@ -286,7 +301,7 @@ func TestLoadWithReferenceInReference(t *testing.T) {
 	require.NotNil(t, doc)
 	err = doc.Validate(loader.Context)
 	require.NoError(t, err)
-	require.Equal(t, "string", doc.Paths["/api/test/ref/in/ref"].Post.RequestBody.Value.Content["application/json"].Schema.Value.Properties["definition_reference"].Value.Type)
+	require.Equal(t, &Types{"string"}, doc.Paths.Value("/api/test/ref/in/ref").Post.RequestBody.Value.Content["application/json"].Schema.Value.Properties["definition_reference"].Value.Type)
 }
 
 func TestLoadWithRecursiveReferenceInLocalReferenceInParentSubdir(t *testing.T) {
@@ -297,10 +312,10 @@ func TestLoadWithRecursiveReferenceInLocalReferenceInParentSubdir(t *testing.T)
 	require.NotNil(t, doc)
 	err = doc.Validate(loader.Context)
 	require.NoError(t, err)
-	require.Equal(t, "object", doc.Paths["/api/test/ref/in/ref"].Post.RequestBody.Value.Content["application/json"].Schema.Value.Properties["definition_reference"].Value.Type)
+	require.Equal(t, &Types{"object"}, doc.Paths.Value("/api/test/ref/in/ref").Post.RequestBody.Value.Content["application/json"].Schema.Value.Properties["definition_reference"].Value.Type)
 }
 
-func TestLoadWithRecursiveReferenceInRefrerenceInLocalReference(t *testing.T) {
+func TestLoadWithRecursiveReferenceInReferenceInLocalReference(t *testing.T) {
 	loader := NewLoader()
 	loader.IsExternalRefsAllowed = true
 	doc, err := loader.LoadFromFile("testdata/refInLocalRef/openapi.json")
@@ -308,8 +323,8 @@ func TestLoadWithRecursiveReferenceInRefrerenceInLocalReference(t *testing.T) {
 	require.NotNil(t, doc)
 	err = doc.Validate(loader.Context)
 	require.NoError(t, err)
-	require.Equal(t, "integer", doc.Paths["/api/test/ref/in/ref"].Post.RequestBody.Value.Content["application/json"].Schema.Value.Properties["data"].Value.Properties["definition_reference"].Value.Properties["ref_prop_part"].Value.Properties["idPart"].Value.Type)
-	require.Equal(t, "int64", doc.Paths["/api/test/ref/in/ref"].Post.RequestBody.Value.Content["application/json"].Schema.Value.Properties["data"].Value.Properties["definition_reference"].Value.Properties["ref_prop_part"].Value.Properties["idPart"].Value.Format)
+	require.Equal(t, &Types{"integer"}, doc.Paths.Value("/api/test/ref/in/ref").Post.RequestBody.Value.Content["application/json"].Schema.Value.Properties["data"].Value.Properties["definition_reference"].Value.Properties["ref_prop_part"].Value.Properties["idPart"].Value.Type)
+	require.Equal(t, "int64", doc.Paths.Value("/api/test/ref/in/ref").Post.RequestBody.Value.Content["application/json"].Schema.Value.Properties["data"].Value.Properties["definition_reference"].Value.Properties["ref_prop_part"].Value.Properties["idPart"].Value.Format)
 }
 
 func TestLoadWithReferenceInReferenceInProperty(t *testing.T) {
@@ -320,7 +335,7 @@ func TestLoadWithReferenceInReferenceInProperty(t *testing.T) {
 	require.NotNil(t, doc)
 	err = doc.Validate(loader.Context)
 	require.NoError(t, err)
-	require.Equal(t, "Problem details", doc.Paths["/api/test/ref/in/ref/in/property"].Post.Responses["401"].Value.Content["application/json"].Schema.Value.Properties["error"].Value.Title)
+	require.Equal(t, "Problem details", doc.Paths.Value("/api/test/ref/in/ref/in/property").Post.Responses.Value("401").Value.Content["application/json"].Schema.Value.Properties["error"].Value.Title)
 }
 
 func TestLoadFileWithExternalSchemaRef(t *testing.T) {
@@ -379,8 +394,8 @@ func TestLoadRequestResponseHeaderRef(t *testing.T) {
 	doc, err := loader.LoadFromData(spec)
 	require.NoError(t, err)
 
-	require.NotNil(t, doc.Paths["/test"].Post.Responses["default"].Value.Headers["X-TEST-HEADER"].Value.Description)
-	require.Equal(t, "testheader", doc.Paths["/test"].Post.Responses["default"].Value.Headers["X-TEST-HEADER"].Value.Description)
+	require.NotNil(t, doc.Paths.Value("/test").Post.Responses.Default().Value.Headers["X-TEST-HEADER"].Value.Description)
+	require.Equal(t, "testheader", doc.Paths.Value("/test").Post.Responses.Default().Value.Headers["X-TEST-HEADER"].Value.Description)
 }
 
 func TestLoadFromDataWithExternalRequestResponseHeaderRemoteRef(t *testing.T) {
@@ -419,8 +434,8 @@ func TestLoadFromDataWithExternalRequestResponseHeaderRemoteRef(t *testing.T) {
 	doc, err := loader.LoadFromDataWithPath(spec, &url.URL{Path: "testdata/testfilename.openapi.json"})
 	require.NoError(t, err)
 
-	require.NotNil(t, doc.Paths["/test"].Post.Responses["default"].Value.Headers["X-TEST-HEADER"].Value.Description)
-	require.Equal(t, "description", doc.Paths["/test"].Post.Responses["default"].Value.Headers["X-TEST-HEADER"].Value.Description)
+	require.NotNil(t, doc.Paths.Value("/test").Post.Responses.Default().Value.Headers["X-TEST-HEADER"].Value.Description)
+	require.Equal(t, "description", doc.Paths.Value("/test").Post.Responses.Default().Value.Headers["X-TEST-HEADER"].Value.Description)
 }
 
 func TestLoadYamlFile(t *testing.T) {
@@ -447,8 +462,8 @@ func TestLoadYamlFileWithExternalPathRef(t *testing.T) {
 	doc, err := loader.LoadFromFile("testdata/pathref.openapi.yml")
 	require.NoError(t, err)
 
-	require.NotNil(t, doc.Paths["/test"].Get.Responses["200"].Value.Content["application/json"].Schema.Value.Type)
-	require.Equal(t, "string", doc.Paths["/test"].Get.Responses["200"].Value.Content["application/json"].Schema.Value.Type)
+	require.NotNil(t, doc.Paths.Value("/test").Get.Responses.Value("200").Value.Content["application/json"].Schema.Value.Type)
+	require.Equal(t, &Types{"string"}, doc.Paths.Value("/test").Get.Responses.Value("200").Value.Content["application/json"].Schema.Value.Type)
 }
 
 func TestResolveResponseLinkRef(t *testing.T) {
@@ -490,7 +505,7 @@ paths:
 	err = doc.Validate(loader.Context)
 	require.NoError(t, err)
 
-	response := doc.Paths[`/users/{id}`].Get.Responses.Get(200).Value
+	response := doc.Paths.Value("/users/{id}").Get.Responses.Status(200).Value
 	link := response.Links[`father`].Value
 	require.NotNil(t, link)
 	require.Equal(t, "getUserById", link.OperationID)
@@ -503,7 +518,7 @@ func TestLinksFromOAISpec(t *testing.T) {
 	require.NoError(t, err)
 	err = doc.Validate(loader.Context)
 	require.NoError(t, err)
-	response := doc.Paths[`/2.0/repositories/{username}/{slug}`].Get.Responses.Get(200).Value
+	response := doc.Paths.Value("/2.0/repositories/{username}/{slug}").Get.Responses.Status(200).Value
 	link := response.Links[`repositoryPullRequests`].Value
 	require.Equal(t, map[string]interface{}{
 		"username": "$response.body#/owner/username",
@@ -607,3 +622,53 @@ servers:
 		})
 	}
 }
+
+func TestReadFromIoReader(t *testing.T) {
+	buffer := bytes.NewReader([]byte(`openapi: 3.0.0
+info:
+  title: An API
+  version: v1
+components:
+  schemas:
+    NewItem:
+      required: [name]
+      properties:
+        name: {type: string}
+        tag: {type: string}
+    ErrorModel:
+      type: object
+      required: [code, message]
+      properties:
+        code: {type: integer}
+        message: {type: string}
+paths:
+  /items:
+    put:
+      description: ''
+      requestBody:
+        required: true
+        content:
+          application/json:
+            schema:
+              $ref: '#/components/schemas/NewItem'
+      responses:
+        default: &defaultResponse # a YAML ref
+          description: unexpected error
+          content:
+            application/json:
+              schema:
+                $ref: '#/components/schemas/ErrorModel'`))
+
+	loader := NewLoader()
+	doc, err := loader.LoadFromIoReader(buffer)
+	require.NoError(t, err)
+
+	err = doc.Validate(loader.Context)
+	require.NoError(t, err)
+}
+
+func TestReadFromIoReader_Nil(t *testing.T) {
+	loader := NewLoader()
+	_, err := loader.LoadFromIoReader(nil)
+	require.EqualError(t, err, "invalid reader: <nil>")
+}
