@@ -153,6 +153,40 @@ var schemaExamples = []schemaExample{
 		},
 	},
 
+	{
+		Title: "PRIMITIVES WITHOUT NULL",
+		Schema: &Schema{
+			Type: &Types{TypeString, TypeBoolean},
+		},
+		AllValid: []interface{}{
+			"",
+			"xyz",
+			true,
+			false,
+		},
+		AllInvalid: []interface{}{
+			1,
+			nil,
+		},
+	},
+
+	{
+		Title: "PRIMITIVES WITH NULL",
+		Schema: &Schema{
+			Type: &Types{TypeNumber, TypeNull},
+		},
+		AllValid: []interface{}{
+			0,
+			1,
+			2.3,
+			nil,
+		},
+		AllInvalid: []interface{}{
+			"x",
+			[]interface{}{},
+		},
+	},
+
 	{
 		Title: "NULLABLE ANYOF",
 		Schema: NewAnyOfSchema(
@@ -230,6 +264,39 @@ var schemaExamples = []schemaExample{
 		},
 	},
 
+	{
+		Title: "ANYOF WITH PARENT CONSTRAINTS",
+		Schema: NewAnyOfSchema(
+			NewObjectSchema().WithRequired([]string{"stringProp"}),
+			NewObjectSchema().WithRequired([]string{"boolProp"}),
+		).WithProperties(map[string]*Schema{
+			"stringProp": NewStringSchema().WithMaxLength(18),
+			"boolProp":   NewBoolSchema(),
+		}),
+		Serialization: map[string]interface{}{
+			"properties": map[string]interface{}{
+				"stringProp": map[string]interface{}{"type": "string", "maxLength": 18},
+				"boolProp":   map[string]interface{}{"type": "boolean"},
+			},
+			"anyOf": []interface{}{
+				map[string]interface{}{"type": "object", "required": []string{"stringProp"}},
+				map[string]interface{}{"type": "object", "required": []string{"boolProp"}},
+			},
+		},
+		AllValid: []interface{}{
+			map[string]interface{}{"stringProp": "valid string value"},
+			map[string]interface{}{"boolProp": true},
+			map[string]interface{}{"stringProp": "valid string value", "boolProp": true},
+		},
+		AllInvalid: []interface{}{
+			1,
+			map[string]interface{}{},
+			map[string]interface{}{"invalidProp": false},
+			map[string]interface{}{"stringProp": "invalid string value"},
+			map[string]interface{}{"stringProp": "invalid string value", "boolProp": true},
+		},
+	},
+
 	{
 		Title:  "BOOLEAN",
 		Schema: NewBoolSchema(),
@@ -476,7 +543,7 @@ var schemaExamples = []schemaExample{
 	{
 		Title: "ARRAY",
 		Schema: &Schema{
-			Type:        "array",
+			Type:        &Types{"array"},
 			MinItems:    2,
 			MaxItems:    Uint64Ptr(3),
 			UniqueItems: true,
@@ -516,10 +583,10 @@ var schemaExamples = []schemaExample{
 	{
 		Title: "ARRAY : items format 'object'",
 		Schema: &Schema{
-			Type:        "array",
+			Type:        &Types{"array"},
 			UniqueItems: true,
 			Items: (&Schema{
-				Type: "object",
+				Type: &Types{"object"},
 				Properties: Schemas{
 					"key1": NewFloat64Schema().NewRef(),
 				},
@@ -573,13 +640,13 @@ var schemaExamples = []schemaExample{
 	{
 		Title: "ARRAY : items format 'object' and object with a property of array type ",
 		Schema: &Schema{
-			Type:        "array",
+			Type:        &Types{"array"},
 			UniqueItems: true,
 			Items: (&Schema{
-				Type: "object",
+				Type: &Types{"object"},
 				Properties: Schemas{
 					"key1": (&Schema{
-						Type:        "array",
+						Type:        &Types{"array"},
 						UniqueItems: true,
 						Items:       NewFloat64Schema().NewRef(),
 					}).NewRef(),
@@ -659,10 +726,10 @@ var schemaExamples = []schemaExample{
 	{
 		Title: "ARRAY : items format 'array'",
 		Schema: &Schema{
-			Type:        "array",
+			Type:        &Types{"array"},
 			UniqueItems: true,
 			Items: (&Schema{
-				Type:        "array",
+				Type:        &Types{"array"},
 				UniqueItems: true,
 				Items:       NewFloat64Schema().NewRef(),
 			}).NewRef(),
@@ -703,13 +770,13 @@ var schemaExamples = []schemaExample{
 	{
 		Title: "ARRAY : items format 'array' and array with object type items",
 		Schema: &Schema{
-			Type:        "array",
+			Type:        &Types{"array"},
 			UniqueItems: true,
 			Items: (&Schema{
-				Type:        "array",
+				Type:        &Types{"array"},
 				UniqueItems: true,
 				Items: (&Schema{
-					Type: "object",
+					Type: &Types{"object"},
 					Properties: Schemas{
 						"key1": NewFloat64Schema().NewRef(),
 					},
@@ -807,7 +874,7 @@ var schemaExamples = []schemaExample{
 	{
 		Title: "OBJECT",
 		Schema: &Schema{
-			Type:     "object",
+			Type:     &Types{"object"},
 			MaxProps: Uint64Ptr(2),
 			Properties: Schemas{
 				"numberProperty": NewFloat64Schema().NewRef(),
@@ -851,10 +918,10 @@ var schemaExamples = []schemaExample{
 	},
 	{
 		Schema: &Schema{
-			Type: "object",
+			Type: &Types{"object"},
 			AdditionalProperties: AdditionalProperties{Schema: &SchemaRef{
 				Value: &Schema{
-					Type: "number",
+					Type: &Types{"number"},
 				},
 			}},
 		},
@@ -879,7 +946,7 @@ var schemaExamples = []schemaExample{
 	},
 	{
 		Schema: &Schema{
-			Type:                 "object",
+			Type:                 &Types{"object"},
 			AdditionalProperties: AdditionalProperties{Has: BoolPtr(true)},
 		},
 		Serialization: map[string]interface{}{
@@ -1366,7 +1433,7 @@ components:
 func TestValidationFailsOnInvalidPattern(t *testing.T) {
 	schema := Schema{
 		Pattern: "[",
-		Type:    "string",
+		Type:    &Types{"string"},
 	}
 
 	err := schema.Validate(context.Background())
@@ -1417,7 +1484,7 @@ enum:
 
 func TestIssue751(t *testing.T) {
 	schema := &Schema{
-		Type:        "array",
+		Type:        &Types{"array"},
 		UniqueItems: true,
 		Items:       NewStringSchema().NewRef(),
 	}
