@@ -79,8 +79,7 @@ type SecurityRequirementsError struct {
 }
 
 func (err *SecurityRequirementsError) Error() string {
-	buff := &bytes.Buffer{}
-	buff.WriteString("security requirements failed: ")
+	buff := bytes.NewBufferString("security requirements failed: ")
 	for i, e := range err.Errors {
 		buff.WriteString(e.Error())
 		if i != len(err.Errors)-1 {
@@ -90,3 +89,9 @@ func (err *SecurityRequirementsError) Error() string {
 
 	return buff.String()
 }
+
+var _ interface{ Unwrap() []error } = SecurityRequirementsError{}
+
+func (err SecurityRequirementsError) Unwrap() []error {
+	return err.Errors
+}
