@@ -8,11 +8,11 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"mime"
 	"mime/multipart"
 	"net/http"
 	"net/url"
+	"reflect"
 	"regexp"
 	"strconv"
 	"strings"
@@ -194,7 +194,7 @@ func defaultContentParameterDecoder(param *openapi3.Parameter, values []string)
 
 	unmarshal := func(encoded string, paramSchema *openapi3.SchemaRef) (decoded interface{}, err error) {
 		if err = json.Unmarshal([]byte(encoded), &decoded); err != nil {
-			if paramSchema != nil && paramSchema.Value.Type != "object" {
+			if paramSchema != nil && !paramSchema.Value.Type.Is("object") {
 				decoded, err = encoded, nil
 			}
 		}
@@ -315,14 +315,14 @@ func decodeValue(dec valueDecoder, param string, sm *openapi3.SerializationMetho
 		return nil, found, errors.New("not implemented: decoding 'not'")
 	}
 
-	if schema.Value.Type != "" {
+	if schema.Value.Type != nil {
 		var decodeFn func(param string, sm *openapi3.SerializationMethod, schema *openapi3.SchemaRef) (interface{}, bool, error)
-		switch schema.Value.Type {
-		case "array":
+		switch {
+		case schema.Value.Type.Is("array"):
 			decodeFn = func(param string, sm *openapi3.SerializationMethod, schema *openapi3.SchemaRef) (interface{}, bool, error) {
 				return dec.DecodeArray(param, sm, schema)
 			}
-		case "object":
+		case schema.Value.Type.Is("object"):
 			decodeFn = func(param string, sm *openapi3.SerializationMethod, schema *openapi3.SchemaRef) (interface{}, bool, error) {
 				return dec.DecodeObject(param, sm, schema)
 			}
@@ -469,6 +469,7 @@ func (d *pathParamDecoder) DecodeObject(param string, sm *openapi3.Serialization
 	if err != nil {
 		return nil, ok, err
 	}
+
 	val, err := makeObject(props, schema)
 	return val, ok, err
 }
@@ -505,7 +506,7 @@ func (d *urlValuesDecoder) DecodePrimitive(param string, sm *openapi3.Serializat
 		return nil, ok, nil
 	}
 
-	if schema.Value.Type == "" && schema.Value.Pattern != "" {
+	if schema.Value.Type == nil && schema.Value.Pattern != "" {
 		return values[0], ok, nil
 	}
 	val, err := parsePrimitive(values[0], schema)
@@ -616,9 +617,12 @@ func (d *urlValuesDecoder) parseValue(v string, schema *openapi3.SchemaRef) (int
 	}
 
 	return parsePrimitive(v, schema)
-
 }
 
+const (
+	urlDecoderDelimiter = "\x1F" // should not conflict with URL characters
+)
+
 func (d *urlValuesDecoder) DecodeObject(param string, sm *openapi3.SerializationMethod, schema *openapi3.SchemaRef) (map[string]interface{}, bool, error) {
 	var propsFn func(url.Values) (map[string]string, error)
 	switch sm.Style {
@@ -646,12 +650,18 @@ func (d *urlValuesDecoder) DecodeObject(param string, sm *openapi3.Serialization
 		propsFn = func(params url.Values) (map[string]string, error) {
 			props := make(map[string]string)
 			for key, values := range params {
-				groups := regexp.MustCompile(fmt.Sprintf("%s\\[(.+?)\\]", param)).FindAllStringSubmatch(key, -1)
-				if len(groups) == 0 {
+				matches := regexp.MustCompile(`\[(.*?)\]`).FindAllStringSubmatch(key, -1)
+				switch l := len(matches); {
+				case l == 0:
 					// A query parameter's name does not match the required format, so skip it.
 					continue
+				case l >= 1:
+					kk := []string{}
+					for _, m := range matches {
+						kk = append(kk, m[1])
+					}
+					props[strings.Join(kk, urlDecoderDelimiter)] = strings.Join(values, urlDecoderDelimiter)
 				}
-				props[groups[0][1]] = values[0]
 			}
 			if len(props) == 0 {
 				// HTTP request does not contain query parameters encoded by rules of style "deepObject".
@@ -662,7 +672,6 @@ func (d *urlValuesDecoder) DecodeObject(param string, sm *openapi3.Serialization
 	default:
 		return nil, false, invalidSerializationMethodErr(sm)
 	}
-
 	props, err := propsFn(d.values)
 	if err != nil {
 		return nil, false, err
@@ -670,17 +679,30 @@ func (d *urlValuesDecoder) DecodeObject(param string, sm *openapi3.Serialization
 	if props == nil {
 		return nil, false, nil
 	}
+	val, err := makeObject(props, schema)
+	if err != nil {
+		return nil, false, err
+	}
 
-	// check the props
 	found := false
 	for propName := range schema.Value.Properties {
 		if _, ok := props[propName]; ok {
 			found = true
 			break
 		}
+
+		if schema.Value.Type.Permits("array") || schema.Value.Type.Permits("object") {
+			for k := range props {
+				path := strings.Split(k, urlDecoderDelimiter)
+				if _, ok := deepGet(val, path...); ok {
+					found = true
+					break
+				}
+			}
+		}
 	}
-	val, err := makeObject(props, schema)
-	return val, found, err
+
+	return val, found, nil
 }
 
 // headerParamDecoder decodes values of header parameters.
@@ -812,7 +834,7 @@ func propsFromString(src, propDelim, valueDelim string) (map[string]string, erro
 	pairs := strings.Split(src, propDelim)
 
 	// When propDelim and valueDelim is equal the source string follow the next rule:
-	// every even item of pairs is a properies's name, and the subsequent odd item is a property's value.
+	// every even item of pairs is a properties's name, and the subsequent odd item is a property's value.
 	if propDelim == valueDelim {
 		// Taking into account the rule above, a valid source string must be splitted by propDelim
 		// to an array with an even number of items.
@@ -845,22 +867,240 @@ func propsFromString(src, propDelim, valueDelim string) (map[string]string, erro
 	return props, nil
 }
 
-// makeObject returns an object that contains properties from props.
-// A value of every property is parsed as a primitive value.
-// The function returns an error when an error happened while parse object's properties.
-func makeObject(props map[string]string, schema *openapi3.SchemaRef) (map[string]interface{}, error) {
-	obj := make(map[string]interface{})
-	for propName, propSchema := range schema.Value.Properties {
-		value, err := parsePrimitive(props[propName], propSchema)
-		if err != nil {
-			if v, ok := err.(*ParseError); ok {
-				return nil, &ParseError{path: []interface{}{propName}, Cause: v}
-			}
-			return nil, fmt.Errorf("property %q: %w", propName, err)
+func deepGet(m map[string]interface{}, keys ...string) (interface{}, bool) {
+	for _, key := range keys {
+		val, ok := m[key]
+		if !ok {
+			return nil, false
+		}
+		if m, ok = val.(map[string]interface{}); !ok {
+			return val, true
 		}
-		obj[propName] = value
 	}
-	return obj, nil
+	return m, true
+}
+
+func deepSet(m map[string]interface{}, keys []string, value interface{}) {
+	for i := 0; i < len(keys)-1; i++ {
+		key := keys[i]
+		if _, ok := m[key]; !ok {
+			m[key] = make(map[string]interface{})
+		}
+		m = m[key].(map[string]interface{})
+	}
+	m[keys[len(keys)-1]] = value
+}
+
+func findNestedSchema(parentSchema *openapi3.SchemaRef, keys []string) (*openapi3.SchemaRef, error) {
+	currentSchema := parentSchema
+	for _, key := range keys {
+		if currentSchema.Value.Type.Includes(openapi3.TypeArray) {
+			currentSchema = currentSchema.Value.Items
+		} else {
+			propertySchema, ok := currentSchema.Value.Properties[key]
+			if !ok {
+				if currentSchema.Value.AdditionalProperties.Schema == nil {
+					return nil, fmt.Errorf("nested schema for key %q not found", key)
+				}
+				currentSchema = currentSchema.Value.AdditionalProperties.Schema
+				continue
+			}
+			currentSchema = propertySchema
+		}
+	}
+	return currentSchema, nil
+}
+
+// makeObject returns an object that contains properties from props.
+func makeObject(props map[string]string, schema *openapi3.SchemaRef) (map[string]interface{}, error) {
+	mobj := make(map[string]interface{})
+
+	for kk, value := range props {
+		keys := strings.Split(kk, urlDecoderDelimiter)
+		if strings.Contains(value, urlDecoderDelimiter) {
+			// don't support implicit array indexes anymore
+			p := pathFromKeys(keys)
+			return nil, &ParseError{path: p, Kind: KindInvalidFormat, Reason: "array items must be set with indexes"}
+		}
+		deepSet(mobj, keys, value)
+	}
+	r, err := buildResObj(mobj, nil, "", schema)
+	if err != nil {
+		return nil, err
+	}
+	result, ok := r.(map[string]interface{})
+	if !ok {
+		return nil, &ParseError{Kind: KindOther, Reason: "invalid param object", Value: result}
+	}
+
+	return result, nil
+}
+
+// example: map[0:map[key:true] 1:map[key:false]] -> [map[key:true] map[key:false]]
+func sliceMapToSlice(m map[string]interface{}) ([]interface{}, error) {
+	var result []interface{}
+
+	keys := make([]int, 0, len(m))
+	for k := range m {
+		key, err := strconv.Atoi(k)
+		if err != nil {
+			return nil, fmt.Errorf("array indexes must be integers: %w", err)
+		}
+		keys = append(keys, key)
+	}
+	max := -1
+	for _, k := range keys {
+		if k > max {
+			max = k
+		}
+	}
+	for i := 0; i <= max; i++ {
+		val, ok := m[strconv.Itoa(i)]
+		if !ok {
+			result = append(result, nil)
+			continue
+		}
+		result = append(result, val)
+	}
+	return result, nil
+}
+
+// buildResObj constructs an object based on a given schema and param values
+func buildResObj(params map[string]interface{}, parentKeys []string, key string, schema *openapi3.SchemaRef) (interface{}, error) {
+	mapKeys := parentKeys
+	if key != "" {
+		mapKeys = append(mapKeys, key)
+	}
+
+	switch {
+	case schema.Value.Type.Is("array"):
+		paramArr, ok := deepGet(params, mapKeys...)
+		if !ok {
+			return nil, nil
+		}
+		t, isMap := paramArr.(map[string]interface{})
+		if !isMap {
+			return nil, &ParseError{path: pathFromKeys(mapKeys), Kind: KindInvalidFormat, Reason: "array items must be set with indexes"}
+		}
+		// intermediate arrays have to be instantiated
+		arr, err := sliceMapToSlice(t)
+		if err != nil {
+			return nil, &ParseError{path: pathFromKeys(mapKeys), Kind: KindInvalidFormat, Reason: fmt.Sprintf("could not convert value map to array: %v", err)}
+		}
+		resultArr := make([]interface{} /*not 0,*/, len(arr))
+		for i := range arr {
+			r, err := buildResObj(params, mapKeys, strconv.Itoa(i), schema.Value.Items)
+			if err != nil {
+				return nil, err
+			}
+			if r != nil {
+				resultArr[i] = r
+			}
+		}
+		return resultArr, nil
+	case schema.Value.Type.Is("object"):
+		resultMap := make(map[string]interface{})
+		additPropsSchema := schema.Value.AdditionalProperties.Schema
+		pp, _ := deepGet(params, mapKeys...)
+		objectParams, ok := pp.(map[string]interface{})
+		if !ok {
+			// not the expected type, but return it either way and leave validation up to ValidateParameter
+			return pp, nil
+		}
+		for k, propSchema := range schema.Value.Properties {
+			r, err := buildResObj(params, mapKeys, k, propSchema)
+			if err != nil {
+				return nil, err
+			}
+			if r != nil {
+				resultMap[k] = r
+			}
+		}
+		if additPropsSchema != nil {
+			// dynamic creation of possibly nested objects
+			for k := range objectParams {
+				r, err := buildResObj(params, mapKeys, k, additPropsSchema)
+				if err != nil {
+					return nil, err
+				}
+				if r != nil {
+					resultMap[k] = r
+				}
+			}
+		}
+
+		return resultMap, nil
+	case len(schema.Value.AnyOf) > 0:
+		return buildFromSchemas(schema.Value.AnyOf, params, parentKeys, key)
+	case len(schema.Value.OneOf) > 0:
+		return buildFromSchemas(schema.Value.OneOf, params, parentKeys, key)
+	case len(schema.Value.AllOf) > 0:
+		return buildFromSchemas(schema.Value.AllOf, params, parentKeys, key)
+	default:
+		val, ok := deepGet(params, mapKeys...)
+		if !ok {
+			// leave validation up to ValidateParameter. here there really is not parameter set
+			return nil, nil
+		}
+		v, ok := val.(string)
+		if !ok {
+			return nil, &ParseError{path: pathFromKeys(mapKeys), Kind: KindInvalidFormat, Value: val, Reason: "path is not convertible to primitive"}
+		}
+		prim, err := parsePrimitive(v, schema)
+		if err != nil {
+			return nil, handlePropParseError(mapKeys, err)
+		}
+
+		return prim, nil
+	}
+}
+
+// buildFromSchemas decodes params with anyOf, oneOf, allOf schemas.
+func buildFromSchemas(schemas openapi3.SchemaRefs, params map[string]interface{}, mapKeys []string, key string) (interface{}, error) {
+	resultMap := make(map[string]interface{})
+	for _, s := range schemas {
+		val, err := buildResObj(params, mapKeys, key, s)
+		if err == nil && val != nil {
+
+			if m, ok := val.(map[string]interface{}); ok {
+				for k, v := range m {
+					resultMap[k] = v
+				}
+				continue
+			}
+
+			if a, ok := val.([]interface{}); ok {
+				if len(a) > 0 {
+					return a, nil
+				}
+				continue
+			}
+
+			// if its a primitive and not nil just return that and let it be validated
+			return val, nil
+		}
+	}
+
+	if len(resultMap) > 0 {
+		return resultMap, nil
+	}
+
+	return nil, nil
+}
+
+func handlePropParseError(path []string, err error) error {
+	if v, ok := err.(*ParseError); ok {
+		return &ParseError{path: pathFromKeys(path), Cause: v}
+	}
+	return fmt.Errorf("property %q: %w", strings.Join(path, "."), err)
+}
+
+func pathFromKeys(kk []string) []interface{} {
+	path := make([]interface{}, 0, len(kk))
+	for _, v := range kk {
+		path = append(path, v)
+	}
+	return path
 }
 
 // parseArray returns an array that contains items from a raw array.
@@ -890,40 +1130,49 @@ func parseArray(raw []string, schemaRef *openapi3.SchemaRef) ([]interface{}, err
 // parsePrimitive returns a value that is created by parsing a source string to a primitive type
 // that is specified by a schema. The function returns nil when the source string is empty.
 // The function panics when a schema has a non-primitive type.
-func parsePrimitive(raw string, schema *openapi3.SchemaRef) (interface{}, error) {
+func parsePrimitive(raw string, schema *openapi3.SchemaRef) (v interface{}, err error) {
 	if raw == "" {
 		return nil, nil
 	}
-	switch schema.Value.Type {
+	for _, typ := range schema.Value.Type.Slice() {
+		if v, err = parsePrimitiveCase(raw, schema, typ); err == nil {
+			return
+		}
+	}
+	return
+}
+
+func parsePrimitiveCase(raw string, schema *openapi3.SchemaRef, typ string) (interface{}, error) {
+	switch typ {
 	case "integer":
 		if schema.Value.Format == "int32" {
 			v, err := strconv.ParseInt(raw, 0, 32)
 			if err != nil {
-				return nil, &ParseError{Kind: KindInvalidFormat, Value: raw, Reason: "an invalid " + schema.Value.Type, Cause: err.(*strconv.NumError).Err}
+				return nil, &ParseError{Kind: KindInvalidFormat, Value: raw, Reason: "an invalid " + typ, Cause: err.(*strconv.NumError).Err}
 			}
 			return int32(v), nil
 		}
 		v, err := strconv.ParseInt(raw, 0, 64)
 		if err != nil {
-			return nil, &ParseError{Kind: KindInvalidFormat, Value: raw, Reason: "an invalid " + schema.Value.Type, Cause: err.(*strconv.NumError).Err}
+			return nil, &ParseError{Kind: KindInvalidFormat, Value: raw, Reason: "an invalid " + typ, Cause: err.(*strconv.NumError).Err}
 		}
 		return v, nil
 	case "number":
 		v, err := strconv.ParseFloat(raw, 64)
 		if err != nil {
-			return nil, &ParseError{Kind: KindInvalidFormat, Value: raw, Reason: "an invalid " + schema.Value.Type, Cause: err.(*strconv.NumError).Err}
+			return nil, &ParseError{Kind: KindInvalidFormat, Value: raw, Reason: "an invalid " + typ, Cause: err.(*strconv.NumError).Err}
 		}
 		return v, nil
 	case "boolean":
 		v, err := strconv.ParseBool(raw)
 		if err != nil {
-			return nil, &ParseError{Kind: KindInvalidFormat, Value: raw, Reason: "an invalid " + schema.Value.Type, Cause: err.(*strconv.NumError).Err}
+			return nil, &ParseError{Kind: KindInvalidFormat, Value: raw, Reason: "an invalid " + typ, Cause: err.(*strconv.NumError).Err}
 		}
 		return v, nil
 	case "string":
 		return raw, nil
 	default:
-		panic(fmt.Sprintf("schema has non primitive type %q", schema.Value.Type))
+		return nil, &ParseError{Kind: KindOther, Value: raw, Reason: "schema has non primitive type " + typ}
 	}
 }
 
@@ -1005,10 +1254,10 @@ func decodeBody(body io.Reader, header http.Header, schema *openapi3.SchemaRef,
 }
 
 func init() {
-	RegisterBodyDecoder("application/json", jsonBodyDecoder)
-	RegisterBodyDecoder("application/json-patch+json", jsonBodyDecoder)
+	RegisterBodyDecoder("application/json", JSONBodyDecoder)
+	RegisterBodyDecoder("application/json-patch+json", JSONBodyDecoder)
 	RegisterBodyDecoder("application/octet-stream", FileBodyDecoder)
-	RegisterBodyDecoder("application/problem+json", jsonBodyDecoder)
+	RegisterBodyDecoder("application/problem+json", JSONBodyDecoder)
 	RegisterBodyDecoder("application/x-www-form-urlencoded", urlencodedBodyDecoder)
 	RegisterBodyDecoder("application/x-yaml", yamlBodyDecoder)
 	RegisterBodyDecoder("application/yaml", yamlBodyDecoder)
@@ -1019,14 +1268,16 @@ func init() {
 }
 
 func plainBodyDecoder(body io.Reader, header http.Header, schema *openapi3.SchemaRef, encFn EncodingFn) (interface{}, error) {
-	data, err := ioutil.ReadAll(body)
+	data, err := io.ReadAll(body)
 	if err != nil {
 		return nil, &ParseError{Kind: KindInvalidFormat, Cause: err}
 	}
 	return string(data), nil
 }
 
-func jsonBodyDecoder(body io.Reader, header http.Header, schema *openapi3.SchemaRef, encFn EncodingFn) (interface{}, error) {
+// JSONBodyDecoder decodes a JSON formatted body. It is public so that is easy
+// to register additional JSON based formats.
+func JSONBodyDecoder(body io.Reader, header http.Header, schema *openapi3.SchemaRef, encFn EncodingFn) (interface{}, error) {
 	var value interface{}
 	dec := json.NewDecoder(body)
 	dec.UseNumber()
@@ -1048,23 +1299,24 @@ func urlencodedBodyDecoder(body io.Reader, header http.Header, schema *openapi3.
 	// Validate schema of request body.
 	// By the OpenAPI 3 specification request body's schema must have type "object".
 	// Properties of the schema describes individual parts of request body.
-	if schema.Value.Type != "object" {
+	if !schema.Value.Type.Is("object") {
 		return nil, errors.New("unsupported schema of request body")
 	}
 	for propName, propSchema := range schema.Value.Properties {
-		switch propSchema.Value.Type {
-		case "object":
+		propType := propSchema.Value.Type
+		switch {
+		case propType.Is("object"):
 			return nil, fmt.Errorf("unsupported schema of request body's property %q", propName)
-		case "array":
+		case propType.Is("array"):
 			items := propSchema.Value.Items.Value
-			if items.Type != "string" && items.Type != "integer" && items.Type != "number" && items.Type != "boolean" {
+			if !(items.Type.Is("string") || items.Type.Is("integer") || items.Type.Is("number") || items.Type.Is("boolean")) {
 				return nil, fmt.Errorf("unsupported schema of request body's property %q", propName)
 			}
 		}
 	}
 
 	// Parse form.
-	b, err := ioutil.ReadAll(body)
+	b, err := io.ReadAll(body)
 	if err != nil {
 		return nil, err
 	}
@@ -1076,27 +1328,60 @@ func urlencodedBodyDecoder(body io.Reader, header http.Header, schema *openapi3.
 	// Make an object value from form values.
 	obj := make(map[string]interface{})
 	dec := &urlValuesDecoder{values: values}
-	for name, prop := range schema.Value.Properties {
-		var (
-			value interface{}
-			enc   *openapi3.Encoding
-		)
-		if encFn != nil {
-			enc = encFn(name)
-		}
-		sm := enc.SerializationMethod()
 
-		if value, _, err = decodeValue(dec, name, sm, prop, false); err != nil {
-			return nil, err
-		}
-		obj[name] = value
+	// Decode schema constructs (allOf, anyOf, oneOf)
+	if err := decodeSchemaConstructs(dec, schema.Value.AllOf, obj, encFn); err != nil {
+		return nil, err
+	}
+	if err := decodeSchemaConstructs(dec, schema.Value.AnyOf, obj, encFn); err != nil {
+		return nil, err
+	}
+	if err := decodeSchemaConstructs(dec, schema.Value.OneOf, obj, encFn); err != nil {
+		return nil, err
+	}
+
+	// Decode properties from the main schema
+	if err := decodeSchemaConstructs(dec, []*openapi3.SchemaRef{schema}, obj, encFn); err != nil {
+		return nil, err
 	}
 
 	return obj, nil
 }
 
+// decodeSchemaConstructs tries to decode properties based on provided schemas.
+// This function is for decoding purposes only and not for validation.
+func decodeSchemaConstructs(dec *urlValuesDecoder, schemas []*openapi3.SchemaRef, obj map[string]interface{}, encFn EncodingFn) error {
+	for _, schemaRef := range schemas {
+		for name, prop := range schemaRef.Value.Properties {
+			value, _, err := decodeProperty(dec, name, prop, encFn)
+			if err != nil {
+				continue
+			}
+			if existingValue, exists := obj[name]; exists && !isEqual(existingValue, value) {
+				return fmt.Errorf("conflicting values for property %q", name)
+			}
+			obj[name] = value
+		}
+	}
+
+	return nil
+}
+
+func isEqual(value1, value2 interface{}) bool {
+	return reflect.DeepEqual(value1, value2)
+}
+
+func decodeProperty(dec valueDecoder, name string, prop *openapi3.SchemaRef, encFn EncodingFn) (interface{}, bool, error) {
+	var enc *openapi3.Encoding
+	if encFn != nil {
+		enc = encFn(name)
+	}
+	sm := enc.SerializationMethod()
+	return decodeValue(dec, name, sm, prop, false)
+}
+
 func multipartBodyDecoder(body io.Reader, header http.Header, schema *openapi3.SchemaRef, encFn EncodingFn) (interface{}, error) {
-	if schema.Value.Type != "object" {
+	if !schema.Value.Type.Is("object") {
 		return nil, errors.New("unsupported schema of request body")
 	}
 
@@ -1145,10 +1430,10 @@ func multipartBodyDecoder(body io.Reader, header http.Header, schema *openapi3.S
 				if anyProperties := schema.Value.AdditionalProperties.Has; anyProperties != nil {
 					switch *anyProperties {
 					case true:
-						//additionalProperties: true
+						// additionalProperties: true
 						continue
 					default:
-						//additionalProperties: false
+						// additionalProperties: false
 						return nil, &ParseError{Kind: KindOther, Cause: fmt.Errorf("part %s: undefined", name)}
 					}
 				}
@@ -1159,7 +1444,7 @@ func multipartBodyDecoder(body io.Reader, header http.Header, schema *openapi3.S
 					return nil, &ParseError{Kind: KindOther, Cause: fmt.Errorf("part %s: undefined", name)}
 				}
 			}
-			if valueSchema.Value.Type == "array" {
+			if valueSchema.Value.Type.Is("array") {
 				valueSchema = valueSchema.Value.Items
 			}
 		}
@@ -1204,7 +1489,7 @@ func multipartBodyDecoder(body io.Reader, header http.Header, schema *openapi3.S
 		if len(vv) == 0 {
 			continue
 		}
-		if prop.Value.Type == "array" {
+		if prop.Value.Type.Is("array") {
 			obj[name] = vv
 		} else {
 			obj[name] = vv[0]
@@ -1216,7 +1501,7 @@ func multipartBodyDecoder(body io.Reader, header http.Header, schema *openapi3.S
 
 // FileBodyDecoder is a body decoder that decodes a file body to a string.
 func FileBodyDecoder(body io.Reader, header http.Header, schema *openapi3.SchemaRef, encFn EncodingFn) (interface{}, error) {
-	data, err := ioutil.ReadAll(body)
+	data, err := io.ReadAll(body)
 	if err != nil {
 		return nil, err
 	}
@@ -1265,7 +1550,6 @@ func zipFileBodyDecoder(body io.Reader, header http.Header, schema *openapi3.Sch
 
 			return nil
 		}()
-
 		if err != nil {
 			return nil, err
 		}
