@@ -5,7 +5,6 @@ import (
 	"context"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"net/http"
 	"net/http/httptest"
 	"testing"
@@ -109,8 +108,10 @@ func getValidationTests(t *testing.T) []*validationTest {
 			wantErrReason: routers.ErrMethodNotAllowed.Error(),
 			// TODO: By HTTP spec, this should have an Allow header with what is allowed
 			// but kin-openapi doesn't provide us the requested method or path, so impossible to provide details
-			wantErrResponse: &ValidationError{Status: http.StatusMethodNotAllowed,
-				Title: routers.ErrMethodNotAllowed.Error()},
+			wantErrResponse: &ValidationError{
+				Status: http.StatusMethodNotAllowed,
+				Title:  routers.ErrMethodNotAllowed.Error(),
+			},
 		},
 		{
 			name: "error - missing body on POST",
@@ -118,8 +119,10 @@ func getValidationTests(t *testing.T) []*validationTest {
 				r: missingBody1,
 			},
 			wantErrBody: "request body has an error: " + ErrInvalidRequired.Error(),
-			wantErrResponse: &ValidationError{Status: http.StatusBadRequest,
-				Title: "request body has an error: " + ErrInvalidRequired.Error()},
+			wantErrResponse: &ValidationError{
+				Status: http.StatusBadRequest,
+				Title:  "request body has an error: " + ErrInvalidRequired.Error(),
+			},
 		},
 		{
 			name: "error - empty body on POST",
@@ -127,8 +130,10 @@ func getValidationTests(t *testing.T) []*validationTest {
 				r: missingBody2,
 			},
 			wantErrBody: "request body has an error: " + ErrInvalidRequired.Error(),
-			wantErrResponse: &ValidationError{Status: http.StatusBadRequest,
-				Title: "request body has an error: " + ErrInvalidRequired.Error()},
+			wantErrResponse: &ValidationError{
+				Status: http.StatusBadRequest,
+				Title:  "request body has an error: " + ErrInvalidRequired.Error(),
+			},
 		},
 
 		//
@@ -141,8 +146,10 @@ func getValidationTests(t *testing.T) []*validationTest {
 				r: noContentType,
 			},
 			wantErrReason: prefixInvalidCT + ` ""`,
-			wantErrResponse: &ValidationError{Status: http.StatusUnsupportedMediaType,
-				Title: "header Content-Type is required"},
+			wantErrResponse: &ValidationError{
+				Status: http.StatusUnsupportedMediaType,
+				Title:  "header Content-Type is required",
+			},
 		},
 		{
 			name: "error - unknown content-type on POST",
@@ -152,8 +159,10 @@ func getValidationTests(t *testing.T) []*validationTest {
 			wantErrReason:      "failed to decode request body",
 			wantErrParseKind:   KindUnsupportedFormat,
 			wantErrParseReason: prefixUnsupportedCT + ` "application/xml"`,
-			wantErrResponse: &ValidationError{Status: http.StatusUnsupportedMediaType,
-				Title: prefixUnsupportedCT + ` "application/xml"`},
+			wantErrResponse: &ValidationError{
+				Status: http.StatusUnsupportedMediaType,
+				Title:  prefixUnsupportedCT + ` "application/xml"`,
+			},
 		},
 		{
 			name: "error - unsupported content-type on POST",
@@ -161,8 +170,10 @@ func getValidationTests(t *testing.T) []*validationTest {
 				r: unsupportedContentType,
 			},
 			wantErrReason: prefixInvalidCT + ` "text/plain"`,
-			wantErrResponse: &ValidationError{Status: http.StatusUnsupportedMediaType,
-				Title: prefixUnsupportedCT + ` "text/plain"`},
+			wantErrResponse: &ValidationError{
+				Status: http.StatusUnsupportedMediaType,
+				Title:  prefixUnsupportedCT + ` "text/plain"`,
+			},
 		},
 		{
 			name: "success - no content-type header required on GET",
@@ -174,7 +185,57 @@ func getValidationTests(t *testing.T) []*validationTest {
 		//
 		// Query strings
 		//
-
+		{
+			name: "empty deepobject query parameter",
+			args: validationArgs{
+				r: newPetstoreRequest(t, http.MethodGet, "/pet/filter", nil),
+			},
+		},
+		{
+			name: "deepobject query parameter type",
+			args: validationArgs{
+				r: newPetstoreRequest(t, http.MethodGet, "/pet/filter?deepFilter[booleans][0]=true&deepFilter[integers][0]=1&deepFilter[strings][0]=foo%26o&deepFilter[numbers][0]=1", nil),
+			},
+		},
+		{
+			name: "error - incorrect deepobject query parameter type bool",
+			args: validationArgs{
+				r: newPetstoreRequest(t, http.MethodGet, "/pet/filter?deepFilter[booleans][0]=notbool", nil),
+			},
+			wantErrParam:   "deepFilter",
+			wantErrBody:    "parameter \"deepFilter\" in query has an error: path booleans.0: value notbool: an invalid boolean: invalid syntax",
+			wantErrParamIn: "query",
+			wantErrResponse: &ValidationError{
+				Status: http.StatusBadRequest,
+				Title:  "parameter \"deepFilter\" in query is invalid: notbool is an invalid boolean",
+			},
+		},
+		{
+			name: "error - incorrect deepobject query parameter type integer",
+			args: validationArgs{
+				r: newPetstoreRequest(t, http.MethodGet, "/pet/filter?deepFilter[integers][0]=1.234", nil),
+			},
+			wantErrParam:   "deepFilter",
+			wantErrBody:    "parameter \"deepFilter\" in query has an error: path integers.0: value 1.234: an invalid integer: invalid syntax",
+			wantErrParamIn: "query",
+			wantErrResponse: &ValidationError{
+				Status: http.StatusBadRequest,
+				Title:  "parameter \"deepFilter\" in query is invalid: 1.234 is an invalid integer",
+			},
+		},
+		{
+			name: "error - incorrect deepobject query parameter type number",
+			args: validationArgs{
+				r: newPetstoreRequest(t, http.MethodGet, "/pet/filter?deepFilter[numbers][0]=aaa", nil),
+			},
+			wantErrParam:   "deepFilter",
+			wantErrBody:    "parameter \"deepFilter\" in query has an error: path numbers.0: value aaa: an invalid number: invalid syntax",
+			wantErrParamIn: "query",
+			wantErrResponse: &ValidationError{
+				Status: http.StatusBadRequest,
+				Title:  "parameter \"deepFilter\" in query is invalid: aaa is an invalid number",
+			},
+		},
 		{
 			name: "error - missing required query string parameter",
 			args: validationArgs{
@@ -184,8 +245,10 @@ func getValidationTests(t *testing.T) []*validationTest {
 			wantErrParamIn: "query",
 			wantErrBody:    `parameter "status" in query has an error: value is required but missing`,
 			wantErrReason:  "value is required but missing",
-			wantErrResponse: &ValidationError{Status: http.StatusBadRequest,
-				Title: `parameter "status" in query is required`},
+			wantErrResponse: &ValidationError{
+				Status: http.StatusBadRequest,
+				Title:  `parameter "status" in query is required`,
+			},
 		},
 		{
 			name: "error - wrong query string parameter type as integer",
@@ -198,8 +261,10 @@ func getValidationTests(t *testing.T) []*validationTest {
 			// So we'd need to look at the inner one which is a KindInvalidFormat. So just check the error body.
 			wantErrBody: `parameter "ids" in query has an error: path 1: value notAnInt: an invalid integer: invalid syntax`,
 			// TODO: Should we treat query params of the wrong type like a 404 instead of a 400?
-			wantErrResponse: &ValidationError{Status: http.StatusBadRequest,
-				Title: `parameter "ids" in query is invalid: notAnInt is an invalid integer`},
+			wantErrResponse: &ValidationError{
+				Status: http.StatusBadRequest,
+				Title:  `parameter "ids" in query is invalid: notAnInt is an invalid integer`,
+			},
 		},
 		{
 			name: "success - ignores unknown query string parameter",
@@ -216,8 +281,10 @@ func getValidationTests(t *testing.T) []*validationTest {
 			wantErrParamIn: "query",
 			wantErrBody:    `parameter "tags" in query has an error: empty value is not allowed`,
 			wantErrReason:  "empty value is not allowed",
-			wantErrResponse: &ValidationError{Status: http.StatusBadRequest,
-				Title: `parameter "tags" in query is not allowed to be empty`},
+			wantErrResponse: &ValidationError{
+				Status: http.StatusBadRequest,
+				Title:  `parameter "tags" in query is not allowed to be empty`,
+			},
 		},
 		{
 			name: "success - non required query string has empty value, but has AllowEmptyValue",
@@ -244,16 +311,18 @@ func getValidationTests(t *testing.T) []*validationTest {
 			},
 			wantErrParam:        "status",
 			wantErrParamIn:      "query",
-			wantErrSchemaReason: "value \"available,sold\" is not one of the allowed values",
+			wantErrSchemaReason: "value is not one of the allowed values [\"available\",\"pending\",\"sold\"]",
 			wantErrSchemaPath:   "/0",
 			wantErrSchemaValue:  "available,sold",
-			wantErrResponse: &ValidationError{Status: http.StatusBadRequest,
-				Title: "value \"available,sold\" is not one of the allowed values",
+			wantErrResponse: &ValidationError{
+				Status: http.StatusBadRequest,
+				Title:  "value is not one of the allowed values [\"available\",\"pending\",\"sold\"]",
 				Detail: "value available,sold at /0 must be one of: available, pending, sold; " +
 					// TODO: do we really want to use this heuristic to guess
 					//  that they're using the wrong serialization?
 					"perhaps you intended '?status=available&status=sold'",
-				Source: &ValidationErrorSource{Parameter: "status"}},
+				Source: &ValidationErrorSource{Parameter: "status"},
+			},
 		},
 		{
 			name: "error - invalid enum value for query string parameter",
@@ -262,13 +331,15 @@ func getValidationTests(t *testing.T) []*validationTest {
 			},
 			wantErrParam:        "status",
 			wantErrParamIn:      "query",
-			wantErrSchemaReason: "value \"watdis\" is not one of the allowed values",
+			wantErrSchemaReason: "value is not one of the allowed values [\"available\",\"pending\",\"sold\"]",
 			wantErrSchemaPath:   "/1",
 			wantErrSchemaValue:  "watdis",
-			wantErrResponse: &ValidationError{Status: http.StatusBadRequest,
-				Title:  "value \"watdis\" is not one of the allowed values",
+			wantErrResponse: &ValidationError{
+				Status: http.StatusBadRequest,
+				Title:  "value is not one of the allowed values [\"available\",\"pending\",\"sold\"]",
 				Detail: "value watdis at /1 must be one of: available, pending, sold",
-				Source: &ValidationErrorSource{Parameter: "status"}},
+				Source: &ValidationErrorSource{Parameter: "status"},
+			},
 		},
 		{
 			name: "error - invalid enum value, allowing commas (without 'perhaps you intended' recommendation)",
@@ -278,14 +349,16 @@ func getValidationTests(t *testing.T) []*validationTest {
 			},
 			wantErrParam:        "kind",
 			wantErrParamIn:      "query",
-			wantErrSchemaReason: "value \"fish,with,commas\" is not one of the allowed values",
+			wantErrSchemaReason: "value is not one of the allowed values [\"dog\",\"cat\",\"turtle\",\"bird,with,commas\"]",
 			wantErrSchemaPath:   "/1",
 			wantErrSchemaValue:  "fish,with,commas",
-			wantErrResponse: &ValidationError{Status: http.StatusBadRequest,
-				Title:  "value \"fish,with,commas\" is not one of the allowed values",
+			wantErrResponse: &ValidationError{
+				Status: http.StatusBadRequest,
+				Title:  "value is not one of the allowed values [\"dog\",\"cat\",\"turtle\",\"bird,with,commas\"]",
 				Detail: "value fish,with,commas at /1 must be one of: dog, cat, turtle, bird,with,commas",
 				// No 'perhaps you intended' because its the right serialization format
-				Source: &ValidationErrorSource{Parameter: "kind"}},
+				Source: &ValidationErrorSource{Parameter: "kind"},
+			},
 		},
 		{
 			name: "success - valid enum value, allowing commas",
@@ -304,13 +377,15 @@ func getValidationTests(t *testing.T) []*validationTest {
 			},
 			wantErrParam:        "x-environment",
 			wantErrParamIn:      "header",
-			wantErrSchemaReason: "value \"watdis\" is not one of the allowed values",
+			wantErrSchemaReason: "value is not one of the allowed values [\"demo\",\"prod\"]",
 			wantErrSchemaPath:   "/",
 			wantErrSchemaValue:  "watdis",
-			wantErrResponse: &ValidationError{Status: http.StatusBadRequest,
-				Title:  "value \"watdis\" is not one of the allowed values",
+			wantErrResponse: &ValidationError{
+				Status: http.StatusBadRequest,
+				Title:  "value is not one of the allowed values [\"demo\",\"prod\"]",
 				Detail: "value watdis at / must be one of: demo, prod",
-				Source: &ValidationErrorSource{Parameter: "x-environment"}},
+				Source: &ValidationErrorSource{Parameter: "x-environment"},
+			},
 		},
 
 		//
@@ -322,27 +397,31 @@ func getValidationTests(t *testing.T) []*validationTest {
 			args: validationArgs{
 				r: newPetstoreRequest(t, http.MethodPost, "/pet", bytes.NewBufferString(`{"status":"watdis"}`)),
 			},
-			wantErrReason:       "doesn't match the schema",
-			wantErrSchemaReason: "value \"watdis\" is not one of the allowed values",
+			wantErrReason:       "doesn't match schema #/components/schemas/PetWithRequired",
+			wantErrSchemaReason: "value is not one of the allowed values [\"available\",\"pending\",\"sold\"]",
 			wantErrSchemaValue:  "watdis",
 			wantErrSchemaPath:   "/status",
-			wantErrResponse: &ValidationError{Status: http.StatusUnprocessableEntity,
-				Title:  "value \"watdis\" is not one of the allowed values",
+			wantErrResponse: &ValidationError{
+				Status: http.StatusUnprocessableEntity,
+				Title:  "value is not one of the allowed values [\"available\",\"pending\",\"sold\"]",
 				Detail: "value watdis at /status must be one of: available, pending, sold",
-				Source: &ValidationErrorSource{Pointer: "/status"}},
+				Source: &ValidationErrorSource{Pointer: "/status"},
+			},
 		},
 		{
 			name: "error - missing required object attribute",
 			args: validationArgs{
 				r: newPetstoreRequest(t, http.MethodPost, "/pet", bytes.NewBufferString(`{"name":"Bahama"}`)),
 			},
-			wantErrReason:       "doesn't match the schema",
+			wantErrReason:       "doesn't match schema #/components/schemas/PetWithRequired",
 			wantErrSchemaReason: `property "photoUrls" is missing`,
 			wantErrSchemaValue:  map[string]string{"name": "Bahama"},
 			wantErrSchemaPath:   "/photoUrls",
-			wantErrResponse: &ValidationError{Status: http.StatusUnprocessableEntity,
+			wantErrResponse: &ValidationError{
+				Status: http.StatusUnprocessableEntity,
 				Title:  `property "photoUrls" is missing`,
-				Source: &ValidationErrorSource{Pointer: "/photoUrls"}},
+				Source: &ValidationErrorSource{Pointer: "/photoUrls"},
+			},
 		},
 		{
 			name: "error - missing required nested object attribute",
@@ -350,13 +429,15 @@ func getValidationTests(t *testing.T) []*validationTest {
 				r: newPetstoreRequest(t, http.MethodPost, "/pet",
 					bytes.NewBufferString(`{"name":"Bahama","photoUrls":[],"category":{}}`)),
 			},
-			wantErrReason:       "doesn't match the schema",
+			wantErrReason:       "doesn't match schema #/components/schemas/PetWithRequired",
 			wantErrSchemaReason: `property "name" is missing`,
 			wantErrSchemaValue:  map[string]string{},
 			wantErrSchemaPath:   "/category/name",
-			wantErrResponse: &ValidationError{Status: http.StatusUnprocessableEntity,
+			wantErrResponse: &ValidationError{
+				Status: http.StatusUnprocessableEntity,
 				Title:  `property "name" is missing`,
-				Source: &ValidationErrorSource{Pointer: "/category/name"}},
+				Source: &ValidationErrorSource{Pointer: "/category/name"},
+			},
 		},
 		{
 			name: "error - missing required deeply nested object attribute",
@@ -364,13 +445,15 @@ func getValidationTests(t *testing.T) []*validationTest {
 				r: newPetstoreRequest(t, http.MethodPost, "/pet",
 					bytes.NewBufferString(`{"name":"Bahama","photoUrls":[],"category":{"tags": [{}]}}`)),
 			},
-			wantErrReason:       "doesn't match the schema",
+			wantErrReason:       "doesn't match schema #/components/schemas/PetWithRequired",
 			wantErrSchemaReason: `property "name" is missing`,
 			wantErrSchemaValue:  map[string]string{},
 			wantErrSchemaPath:   "/category/tags/0/name",
-			wantErrResponse: &ValidationError{Status: http.StatusUnprocessableEntity,
+			wantErrResponse: &ValidationError{
+				Status: http.StatusUnprocessableEntity,
 				Title:  `property "name" is missing`,
-				Source: &ValidationErrorSource{Pointer: "/category/tags/0/name"}},
+				Source: &ValidationErrorSource{Pointer: "/category/tags/0/name"},
+			},
 		},
 		{
 			name: "error - wrong attribute type",
@@ -378,30 +461,35 @@ func getValidationTests(t *testing.T) []*validationTest {
 				r: newPetstoreRequest(t, http.MethodPost, "/pet",
 					bytes.NewBufferString(`{"name":"Bahama","photoUrls":"http://cat"}`)),
 			},
-			wantErrReason:       "doesn't match the schema",
-			wantErrSchemaReason: "field must be set to array or not be present",
+			wantErrReason:       "doesn't match schema #/components/schemas/PetWithRequired",
+			wantErrSchemaReason: "value must be an array",
 			wantErrSchemaPath:   "/photoUrls",
-			wantErrSchemaValue:  "string",
+			wantErrSchemaValue:  "http://cat",
 			// TODO: this shouldn't say "or not be present", but this requires recursively resolving
 			//  innerErr.JSONPointer() against e.RequestBody.Content["application/json"].Schema.Value (.Required, .Properties)
-			wantErrResponse: &ValidationError{Status: http.StatusUnprocessableEntity,
-				Title:  "field must be set to array or not be present",
-				Source: &ValidationErrorSource{Pointer: "/photoUrls"}},
+			wantErrResponse: &ValidationError{
+				Status: http.StatusUnprocessableEntity,
+				Title:  "value must be an array",
+				Source: &ValidationErrorSource{Pointer: "/photoUrls"},
+			},
 		},
 		{
 			name: "error - missing required object attribute from allOf required overlay",
 			args: validationArgs{
 				r: newPetstoreRequest(t, http.MethodPost, "/pet2", bytes.NewBufferString(`{"name":"Bahama"}`)),
 			},
-			wantErrReason:             "doesn't match the schema",
+			wantErrReason:             "doesn't match schema",
 			wantErrSchemaPath:         "/",
 			wantErrSchemaValue:        map[string]string{"name": "Bahama"},
+			wantErrSchemaReason:       `doesn't match all schemas from "allOf"`,
 			wantErrSchemaOriginReason: `property "photoUrls" is missing`,
 			wantErrSchemaOriginValue:  map[string]string{"name": "Bahama"},
 			wantErrSchemaOriginPath:   "/photoUrls",
-			wantErrResponse: &ValidationError{Status: http.StatusUnprocessableEntity,
+			wantErrResponse: &ValidationError{
+				Status: http.StatusUnprocessableEntity,
 				Title:  `property "photoUrls" is missing`,
-				Source: &ValidationErrorSource{Pointer: "/photoUrls"}},
+				Source: &ValidationErrorSource{Pointer: "/photoUrls"},
+			},
 		},
 		{
 			name: "success - ignores unknown object attribute",
@@ -437,8 +525,10 @@ func getValidationTests(t *testing.T) []*validationTest {
 			wantErrParamIn: "path",
 			wantErrBody:    `parameter "petId" in path has an error: value is required but missing`,
 			wantErrReason:  "value is required but missing",
-			wantErrResponse: &ValidationError{Status: http.StatusBadRequest,
-				Title: `parameter "petId" in path is required`},
+			wantErrResponse: &ValidationError{
+				Status: http.StatusBadRequest,
+				Title:  `parameter "petId" in path is required`,
+			},
 		},
 		{
 			name: "error - wrong path param type",
@@ -450,8 +540,10 @@ func getValidationTests(t *testing.T) []*validationTest {
 			wantErrParseKind:   KindInvalidFormat,
 			wantErrParseValue:  "NotAnInt",
 			wantErrParseReason: "an invalid integer",
-			wantErrResponse: &ValidationError{Status: http.StatusNotFound,
-				Title: `resource not found with "petId" value: NotAnInt`},
+			wantErrResponse: &ValidationError{
+				Status: http.StatusNotFound,
+				Title:  `resource not found with "petId" value: NotAnInt`,
+			},
 		},
 		{
 			name: "success - normal case, with path params",
@@ -470,7 +562,6 @@ func TestValidationHandler_validateRequest(t *testing.T) {
 
 			h, err := buildValidationHandler(tt)
 			req.NoError(err)
-
 			err = h.validateRequest(tt.args.r)
 			req.Equal(tt.wantErr, err != nil)
 
@@ -542,7 +633,7 @@ func TestValidationErrorEncoder(t *testing.T) {
 			req.NoError(err)
 
 			err = h.validateRequest(tt.args.r)
-			req.Equal(tt.wantErr, err != nil)
+			req.Equal(tt.wantErr, err != nil, "wantError: %v", tt.wantErr)
 
 			if err != nil {
 				encoder.Encode(tt.args.r.Context(), err, httptest.NewRecorder())
@@ -556,7 +647,7 @@ func TestValidationErrorEncoder(t *testing.T) {
 
 func buildValidationHandler(tt *validationTest) (*ValidationHandler, error) {
 	if tt.fields.File == "" {
-		tt.fields.File = "fixtures/petstore.json"
+		tt.fields.File = "testdata/fixtures/petstore.json"
 	}
 	h := &ValidationHandler{
 		Handler:      tt.fields.Handler,
@@ -600,7 +691,7 @@ func runTest_ServeHTTP(t *testing.T, handler http.Handler, encoder ErrorEncoder,
 	h := &ValidationHandler{
 		Handler:      handler,
 		ErrorEncoder: encoder,
-		File:         "fixtures/petstore.json",
+		File:         "testdata/fixtures/petstore.json",
 	}
 	err := h.Load()
 	require.NoError(t, err)
@@ -612,7 +703,7 @@ func runTest_ServeHTTP(t *testing.T, handler http.Handler, encoder ErrorEncoder,
 func runTest_Middleware(t *testing.T, handler http.Handler, encoder ErrorEncoder, req *http.Request) *http.Response {
 	h := &ValidationHandler{
 		ErrorEncoder: encoder,
-		File:         "fixtures/petstore.json",
+		File:         "testdata/fixtures/petstore.json",
 	}
 	err := h.Load()
 	require.NoError(t, err)
@@ -656,10 +747,10 @@ func TestValidationHandler_ServeHTTP(t *testing.T) {
 		encoder := &ValidationErrorEncoder{Encoder: (ErrorEncoder)(DefaultErrorEncoder)}
 		resp := runTest_ServeHTTP(t, handler, encoder.Encode, r)
 
-		body, err := ioutil.ReadAll(resp.Body)
+		body, err := io.ReadAll(resp.Body)
 		require.NoError(t, err)
 		require.Equal(t, http.StatusUnprocessableEntity, resp.StatusCode)
-		require.Equal(t, "[422][][] field must be set to array or not be present [source pointer=/photoUrls]", string(body))
+		require.Equal(t, "[422][][] value must be an array [source pointer=/photoUrls]", string(body))
 	})
 }
 
@@ -698,9 +789,9 @@ func TestValidationHandler_Middleware(t *testing.T) {
 		encoder := &ValidationErrorEncoder{Encoder: (ErrorEncoder)(DefaultErrorEncoder)}
 		resp := runTest_Middleware(t, handler, encoder.Encode, r)
 
-		body, err := ioutil.ReadAll(resp.Body)
+		body, err := io.ReadAll(resp.Body)
 		require.NoError(t, err)
 		require.Equal(t, http.StatusUnprocessableEntity, resp.StatusCode)
-		require.Equal(t, "[422][][] field must be set to array or not be present [source pointer=/photoUrls]", string(body))
+		require.Equal(t, "[422][][] value must be an array [source pointer=/photoUrls]", string(body))
 	})
 }
